%!

% Functions for drawing circles and arcs.

% include files
(standard.inc) run

%------------------
/draw-circle
%------------------
% Input: [x y r], the centre and radius of the circle which we want to draw, and the colour 
% [r g b].
% Output: none.
% Draws the circle in the given colour.
{
  3 dict begin

  gsave
  aload pop setrgbcolor		% set colour to r g b
  aload pop			% stack: x y r 
  /r exch def
  /y exch def
  /x exch def
  newpath
  % x r add y moveto		% move to the point at 0 degrees on the circle
  x y r 0 360 arc 		% draw the circle starting at this point
  stroke
  grestore

  end
} def

%------------------
/draw-arc
%------------------
% Input: [x y r], the centre and radius of the arc which we want to draw, a1 and a2, the 
% starting and ending angles, and the colour [r g b].
% Output: none.
% Draws an arc in the given colour.
{
  3 dict begin

  gsave
  aload pop setrgbcolor		% set colour to r g b

  % get the rest of the input
  /a2 exch def
  /a1 exch def
  aload pop			% stack: x y r 
  /r exch def
  /y exch def
  /x exch def

  newpath
  % x y moveto
  % a1 rotate
  % r 0 moveto			% move to the starting point on the arc
  % x y r 0 a1 a2 add arc 	% draw the arc from this point
  x y r a1 a2 arc 		% draw the arc 
  
  stroke
  grestore

  end
} def

%------------------
/centre
%------------------
% Input: [x y r], the co-ordinates of a circle.
% Output: [x y], the centre of the circle.
{
  aload pop pop 		% stack: x y
  [ 3 1 roll ]		% stack: [ x y ]
} def

%------------------
/radius
%------------------
% Input: [x y r], the co-ordinates of a circle.
% Output: r, the radius of the circle.
{
  aload pop 		% stack: x y r
  3 1 roll			% stack: r x y
  pop pop			% stack: r
} def

