%!

% Functions for drawing a line given its homogeneous co-ordinates.

(standard.inc) run

%------------------
/draw-line
%------------------
% Input: [A B C], the homogeneous co-ordinates of the line we want to draw, the co-ordinates
% of the lower left and upper right corners of the bounding box outside of which to clip the
% line, [llx lly urx ury], and the colour, [r g b].
% Output: none.
% Draws the line (as much of it that fits on the screen) in the given colour.
{
  3 dict begin

  gsave
  aload pop setrgbcolor		% set colour to r g b
  aload pop 
  /ury exch def
  /urx exch def
  /lly exch def
  /llx exch def
  /ABC exch def			% get the coefficients of the line

  % get the corners of the bounding box
  /P0 [llx lly] def
  /P1 [urx lly] def
  /P2 [urx ury] def
  /P3 [llx ury] def

  % get the intersection points of the line with the boundaries
  [P0 P1 P2 P3] ABC get-intercepts

  dup /n exch length def	% get the number of points of intersection
  aload pop	% break the array into separate points 
  n 2 eq 
  {
    % draw the line across the bounding box
    newpath
    aload pop moveto
    aload pop lineto
    stroke
  } if
  n 1 eq
  {
    pop	% just throw away this point 
  } if

  grestore

  end
} def

%------------------
/draw-line2
%------------------
% Input: The homogeneous co-ordinates [A B C] of the line we want to draw, and the colour 
% [r g b].
% Draws the line (as much of it that fits on the screen) in the given colour.
{
  3 dict begin

  gsave
  aload pop setrgbcolor		% set colour to r g b
  /ABC exch def			% get the coefficients of the line

  /margin 25 def
  
  % the page dimensions in default co-ordinates
  /page-width 610 def
  /page-height 790 def

  % the default co-ordinates of the lower left and upper right corners
  /llx margin def
  /lly margin def
  /urx page-width margin sub def
  /ury page-height margin sub def

  % the page dimensions in user co-ordinates
  /D matrix defaultmatrix def
  /P0 [ llx lly D transform itransform ] def
  /P1 [ urx lly D transform itransform ] def
  /P2 [ urx ury D transform itransform ] def
  /P3 [ llx ury D transform itransform ] def

  % get the intersection points of the line with the page boundaries
  [P0 P1 P2 P3] ABC get-intercepts

  dup /n exch length def	% get the number of points of intersection
  aload pop	% break the array into separate points 
  n 2 eq 
  {
    % draw the line across the page
    newpath
    aload pop moveto
    aload pop lineto
    stroke
  } if
  n 1 eq
  {
    pop	% just throw away this point 
  } if

  grestore

  end
} def

%------------------
/draw-line3
%------------------
% Input: The homogeneous co-ordinates [A B C] of the line we want to draw, the size of the 
% margin to leave (in points), and the colour [r g b].
% Draws the line (as much of it that fits on the screen) in the given colour.
{
  3 dict begin

  /colour exch def
  /margin exch def
  /ABC exch def			% get the coefficients of the line

  % the page dimensions in default co-ordinates
  /page-width 610 def
  /page-height 790 def

  % the default co-ordinates of the lower left and upper right corners
  /llx margin def
  /lly margin def
  /urx page-width margin sub def
  /ury page-height margin sub def

  /D matrix defaultmatrix def

  ABC 
  [
    llx lly D transform itransform	% co-ordinates of lower left in user co-ordinates
    urx ury D transform itransform	% co-ordinates of upper right in user co-ordinates
  ] 
  colour draw-line 

  end
} def

%------------------
/get-intercepts
%------------------
% Input: An array containing the four corners of a rectangle, in perimeter order, and an array 
% [A B C] defining the equation of a line in the current co-ordinate system.
% Output: An array of the 0, 1, or 2 points of intersection of this line with the rectangle.
{ 
  8 dict begin	% keep internal variables local

  % get the coefficients of the equation of the line from stack
  /ABC exch def

  % get the co-ordinates of the corners of the rectangle
  aload pop  
  /P3 exch def
  /P2 exch def
  /P1 exch def
  /P0 exch def

  [	% begin the return array

  % test each side of the square for intersection points
  P0 P1 ABC test-side
  /test-return exch def	% return value from test-side
  test-return 1 eq	% one point of intersection
  {
    P0 P1 ABC get-intercept	% add intercept to array
  }
  { 
    test-return 2 eq	% the far endpoint is the intercept
    test-return 3 eq or	% the line coincides with the side
    {
      P1		% add the far endpoint to the array
    } if
  } ifelse

  P1 P2 ABC test-side
  /test-return exch def	% return value from test-side
  test-return 1 eq	% one point of intersection
  {
    P1 P2 ABC get-intercept	% add intercept to array
  }
  { 
    test-return 2 eq	% the far endpoint is the intercept
    test-return 3 eq or	% the line coincides with the side
    {
      P2		% add the far endpoint to the array
    } if
  } ifelse

  P2 P3 ABC test-side
  /test-return exch def	% return value from test-side
  test-return 1 eq	% one point of intersection
  {
    P2 P3 ABC get-intercept	% add intercept to array
  }
  { 
    test-return 2 eq	% the far endpoint is the intercept
    test-return 3 eq or	% the line coincides with the side
    {
      P3		% add the far endpoint to the array
    } if
  } ifelse

  P3 P0 ABC test-side
  /test-return exch def	% return value from test-side
  test-return 1 eq	% one point of intersection
  {
    P3 P0 ABC get-intercept	% add intercept to array
  }
  { 
    test-return 2 eq	% the far endpoint is the intercept
    test-return 3 eq or	% the line coincides with the side
    {
      P0		% add the far endpoint to the array
    } if
  } ifelse

  ]	% close array

  end
} def

%------------------
/eval-line
%------------------
% Input: A point [x y] and the array [A B C] of three numbers that % define the line.
% Output: The value Ax + By + C.
{
  5 dict
  begin
  aload pop	% break up the array [A B C] into its elements
  /C exch def
  /B exch def
  /A exch def
  aload pop	% break the point into its co-ordinates
  /y exch def
  /x exch def
  A x mul B y mul C add add	% Ax + By + C
  end
} def

%------------------
/get-intercept
%------------------
% Input: Two points P and Q, and a line L, given by [A B C].
% Output: The point of intersection of the line segment PQ and the
% line L.
% Requirement: That the line L intersects the line PQ (though not 
% necessarily the line segment PQ).
{
  11 dict begin 
  aload pop	% break up the array [A B C] into its elements
  /C exch def
  /B exch def
  /A exch def
  aload pop	% break the points P and Q into their co-ordinates
  /Qy exch def
  /Qx exch def
  aload pop
  /Py exch def
  /Px exch def
  /LP A Px mul B Py mul C add add def 	% APx + BPy + C
  /LQ A Qx mul B Qy mul C add add def	% AQx + BQy + C
  LP LQ eq
  { 
    (Can't evaluate: PQ and L are either coincident or parallel)
    exit 	% Note: I don't know anything about error handling in Postscript
  }
  {
    % interpolate to find intercept R
    /i LP LP LQ sub div def
    /Rx Px Qx Px sub i mul add def	% Rx = Px + (Qx - Px) x i
    /Ry Py Qy Py sub i mul add def	% Ry = Py + (Qy - Py) x i
    [Rx Ry]	% the return value
  } ifelse
  end
} def

%------------------
/test-side
%------------------
% Input: Points P and Q, and line L given by [A B C].
% Output: 0, if line L does not intersect the segment PQ, 
% 1, if the line intersects it at just one interior point, 
% 2, if the line intersects the segment at Q,
% and 3, if the line is coincident with the line segment.
{
  10 dict
  begin
  /ABC exch def		% the array [A B C]
  /Q exch def		% the point Q
  /P exch def		% the point P
  /A ABC 0 get def
  /B ABC 1 get def
  /L1 P ABC eval-line def
  /L2 Q ABC eval-line def

  A 0 eq B 0 eq and
  {
    % there is no line if A and B are both 0
    0	% return 0
    exit
  } if

  L1 L2 mul 0 lt 	% L1 x L2 < 0
  { 
    % L1 and L2 have different signs, therefore the line crosses this side
    1	% return 1
  }
  { % L1 x L2 >= 0
    L1 L2 mul 0 gt	% L1 x L2 > 0
    {
      % L1 and L2 have the same sign, therefore the line does not cross
      % this side
      0	% return 0
    }
    % assertion: either L1 or L2 is zero
    { % L1 x L2 = 0
      L1 0 eq
      % L1 and L2 are both 0 -> the line overlaps the side PQ
      % L1 = 0 and L2 <> 0 -> the line meets this side at P
      % L1 <> 0 and L2 = 0 -> the line meets this side at Q
      {	
        L2 0 eq
        {
          3 % return 3, because the line overlaps the side
        }
        { % L2 <> 0
          0	% return 0, because we are not taking P to be a part of the side
        } ifelse
      }
      { % L1 <> 0
        2 	% return 2, because the line meets the side at Q
      } ifelse	% L1 = 0
    } ifelse	% L1 x L2 > 0
  } ifelse	% L1 x L2 < 0
  end
} def  


