%------------------
/co-ords
%------------------
% Returns the co-ordinates of a point [x y].
% Input: A point in array form: [x y].
% Output: The x and y co-ordinates of the point, x y.
{ aload pop } def

%------------------
/wrap-co-ords
%------------------
% Reverses the co-ords function.
% Input: The x and y co-ordinates of a point, x y.
% Output: The point in array form: [x y].
{ [ 3 1 roll ] } def

%-----------------
/conjugate
%-----------------
% Input: A complex number z = [x y].
% Output: The complex conjugate of the given point, [x -y].
{
  [ exch aload pop neg ]
} def

%------------------
/mark-point
%------------------
% Marks the point (x, y) with a dot of the given colour.
% Input: a point [x y] and a colour, [r g b].
% Output: none.
{
  gsave
  aload pop setrgbcolor 	% set colour to r g b
  3 currentlinewidth mul setlinewidth
  newpath
  co-ords moveto			% move to the given point
  currentpoint currentlinewidth 0 360 arc % draw a circle around that point
  fill
  grestore
} def

%------------------
/mark-right-angle
%------------------
% Marks the angle at point (x, y) with a right-angle symbol in the given colour.
% Input: a point [x y], an angle a, the length of the symbol i, and a colour, [r g b].
% Output: none.
{
  2 dict begin
  gsave
  aload pop setrgbcolor 	% set colour to r g b
  /i exch def			% the length of the symbol
  /a exch def			% the angle at which to begin drawing
  newpath
  co-ords moveto			% move to the given point
  a rotate				% set the orientation
  i 0 rmoveto
  0 i rlineto
  i neg 0 rlineto
  stroke
  grestore
  end
} def

